#! /usr/bin/python3

import os
import argparse
import socket
from os.path import join
import json

import paho.mqtt.client as mqtt
import yaml

version = "0.1.0"

def xdg_config_dir():
    config_home = os.getenv("XDG_CONFIG_HOME",
                            os.path.expanduser("~/.config/"))
    return os.path.join(config_home, "lirc2mqtt")

def connect(path):
    lirc = socket.socket(socket.AF_UNIX)
    try:
        lirc.connect(path)
    except socket.error:
        sys.stderr.write("lirc: unable to open socket\n")
    
    return lirc.makefile()

def parse(line):
    return line.rstrip().split(" ")[2]

def bind_layer(layers, key):
    layer = layers["_"].copy()
    layer.update(layers[key])
    
    return layer

def loop(lirc_path, mqtt_client, init, layers):
    lirc = connect(lirc_path)

    if init in layers:
        binding = bind_layer(layers, init)
    else:
        binding = bind_layer(layers, "_")
    
    for line in lirc:
        key = parse(line)
        
        if key in layers:
            binding = bind_layer(layers, key)
        
        if key not in binding:
            print(None)
            continue
        
        action = binding[key]
        messages = []
        if type(action) is str:
            messages.append((action, {}))
        elif type(action) is dict:
            for topic, data in action.items():
                if data is None:
                    messages.append((topic, {}))
                else:
                    messages.append((topic, action[topic]))
        
        for (topic, data) in messages:
            print(key, topic, data)
            mqtt_client.publish(topic, json.dumps(data))

if __name__=="__main__":
    parser = argparse.ArgumentParser(description='lirc2mqtt -- LIRC to MQTT Gateway')
    parser.add_argument('--version', action='version', version='%(prog)s ' + version)
    parser.add_argument("-b", "--host",
                        dest="host",
                        default="localhost",
                        help="MQTT broker address")
    parser.add_argument("-c", "--config",
                        dest="config",
                        default=join(xdg_config_dir(), "configuration.yml"),
                        help="Configuration file")
    parser.add_argument("--lirc",
                        dest="lirc",
                        default="/dev/lircd",
                        help="Path to LIRC socket")
    parser.add_argument("--init",
                        dest="init",
                        default="_",
                        help="Layer initially selected")
    args = parser.parse_args()
    
    with open(args.config) as f:
        config = yaml.safe_load(f)
    
    client = mqtt.Client()
    client.connect(args.host)
    client.loop_start()
    
    loop(args.lirc, client, args.init, config)
    
